@@ -393,6 +393,7 @@ void CAlphIO::XML_StartElement(void *userData, const XML_Char *name, const XML_C
     Me->InputInfo.m_iCharacters = 1; // Start at 1 as 0 is the root node symbol
     Me->InputInfo.m_pBaseGroup = 0;
     Me->bFirstGroup = true;
+    Me->iGroupIdx = 0;
     while(*atts != 0) {
       if(strcmp(*atts, "name") == 0) {
         atts++;
@@ -401,7 +402,8 @@ void CAlphIO::XML_StartElement(void *userData, const XML_Char *name, const XML_C
       }
       atts += 2;
     }
-
+    Me->InputInfo.m_iConversionID = 0;
+    
     return;
   }
 
@@ -520,7 +522,8 @@ void CAlphIO::XML_StartElement(void *userData, const XML_Char *name, const XML_C
 
   if(strcmp(name, "group") == 0) {
     SGroupInfo *pNewGroup(new SGroupInfo);
-    pNewGroup->iColour = 0;
+    pNewGroup->iColour = (Me->iGroupIdx % 3) + 110; 
+    ++Me->iGroupIdx;
 
     if(Me->bFirstGroup) {
       pNewGroup->bVisible = false;
@@ -578,6 +581,19 @@ void CAlphIO::XML_StartElement(void *userData, const XML_Char *name, const XML_C
     return;
   }
 
+  if(!strcmp(name, "conversionmode")) {
+    while(*atts != 0) {
+      if(strcmp(*atts, "id") == 0) {
+        atts++;
+        Me->InputInfo.m_iConversionID = atoi(*atts);
+        atts--;
+      }
+      atts += 2;
+    }
+
+    return;
+  }
+
   // Special characters for character composition
   if(strcmp(name, "convert") == 0) {
     while(*atts != 0) {
