@@ -22,6 +22,7 @@ static char THIS_FILE[] = __FILE__;
 
 static SModuleSettings sSettings[] = {
   {LP_ZOOMSTEPS, T_LONG, 1, 63, 1, 1, "Zoom steps"},
+  {LP_RIGHTZOOM, T_LONG, 1024, 10240, 1024, 1024, "Right zoom"},
   {LP_B, T_LONG, 2, 10, 1, 1, "Number of boxes"},
   {LP_S, T_LONG, 0, 256, 1, 1, "Safety margin"},
   {LP_R, T_LONG, -400, 400, 1, 10, "Box non-uniformity"},
@@ -54,7 +55,7 @@ CDasherButtons::~CDasherButtons()
 void CDasherButtons::SetupBoxes()
 {
   
-    int iDasherY(GetLongParameter(LP_MAX_Y));
+  int iDasherY(GetLongParameter(LP_MAX_Y));
 
 
   if(m_iStyle == 3) {
@@ -334,7 +335,7 @@ bool CDasherButtons::DecorateView(CDasherView *pView) {
 }
  
 
-void CDasherButtons::KeyDown(int iTime, int iId, CDasherModel *pModel) {
+void CDasherButtons::KeyDown(int iTime, int iId, CDasherModel *pModel, CUserLogBase *pUserLog) {
 
   if(m_bMenu) {
     switch(iId) {
@@ -395,14 +396,14 @@ void CDasherButtons::KeyDown(int iTime, int iId, CDasherModel *pModel) {
 
 }
 
-bool CDasherButtons::Timer(int Time, CDasherView *m_pDasherView, CDasherModel *m_pDasherModel) {
+bool CDasherButtons::Timer(int Time, CDasherView *m_pDasherView, CDasherModel *m_pDasherModel, Dasher::VECTOR_SYMBOL_PROB *pAdded, int *pNumDeleted) {
   bool m_bOldHighlight(m_bHighlight);
   m_bHighlight = (Time - m_iLastTime < 200);
 
   if(m_bOldHighlight != m_bHighlight)
     m_bDecorationChanged = true;
 
-  return m_pDasherModel->Tap_on_display(0, 0, Time, 0, 0);
+  return m_pDasherModel->UpdatePosition(0, 0, Time, pAdded, pNumDeleted);
 }
 
 void CDasherButtons::HandleEvent(Dasher::CEvent * pEvent) {
