@@ -19,6 +19,7 @@ using namespace std;
 #include "LanguageModelling/WordLanguageModel.h"
 #include "LanguageModelling/DictLanguageModel.h"
 #include "LanguageModelling/MixtureLanguageModel.h"
+#include "NodeCreationManager.h"
 
 using namespace Dasher;
 using namespace std;
@@ -37,14 +38,23 @@ static char THIS_FILE[] = __FILE__;
 
 // CDasherModel
 
-CDasherModel::CDasherModel(CEventHandler *pEventHandler, CSettingsStore *pSettingsStore, CDasherInterfaceBase *pDashIface, CAlphIO *pAlphIO, bool bGameMode, const std::string &strGameModeText)
-:CDasherComponent(pEventHandler, pSettingsStore), m_pDasherInterface(pDashIface), m_Root(0), total_nats(0.0), 
-m_pLanguageModel(NULL), m_pcAlphabet(NULL), m_Rootmin(0), m_Rootmax(0), m_Rootmin_min(0),
-m_Rootmax_max(0), m_dAddProb(0.0), m_dMaxRate(0.0) {
+CDasherModel::CDasherModel(CEventHandler *pEventHandler, CSettingsStore *pSettingsStore, CNodeCreationManager *pNCManager, CDasherInterfaceBase *pDashIface, bool bGameMode, const std::string &strGameModeText)
+:CDasherComponent(pEventHandler, pSettingsStore), m_pDasherInterface(pDashIface) {
 
+  m_Root = 0;
+  m_pLanguageModel =NULL; 
+
+  m_Rootmin = 0;
+  m_Rootmax = 0;
+  m_Rootmin_min = 0;
+  m_Rootmax_max = 0;
+  m_dAddProb = 0.0;
+  m_dMaxRate = 0.0;
   m_iTargetOffset = 0;
-
+  m_dTotalNats = 0.0;
   m_bGameMode = bGameMode;
+ 
+  // TODO: Need to rationalise the require conversion methods
 
 #ifdef JAPANESE
   m_bRequireConversion = true;
@@ -57,75 +67,18 @@ m_Rootmax_max(0), m_dAddProb(0.0), m_dMaxRate(0.0) {
   // Set max bitrate in the FrameRate class
   m_dMaxRate = GetLongParameter(LP_MAX_BITRATE) / 100.0;
   m_fr.SetMaxBitrate(m_dMaxRate);
-
-  // Convert the full alphabet to a symbolic representation for use in the language model
-
-  // -- put all this in a separate method
-  // TODO: Think about having 'prefered' values here, which get
-  // retrieved by DasherInterfaceBase and used to set parameters
-
-  // TODO: We might get a different alphabet to the one we asked for -
-  // if this is the case then the parameter value should be updated,
-  // but not in such a way that it causes everything to be rebuilt.
-
-  CAlphIO::AlphInfo oAlphInfo = pAlphIO->GetInfo(GetStringParameter(SP_ALPHABET_ID));
-  m_pcAlphabet = new CAlphabet(oAlphInfo);
-
-  SetStringParameter(SP_TRAIN_FILE, m_pcAlphabet->GetTrainingFile());
-  SetStringParameter(SP_DEFAULT_COLOUR_ID, m_pcAlphabet->GetPalette());
-
-  if(GetLongParameter(LP_ORIENTATION) == Dasher::Opts::AlphabetDefault)
-    SetLongParameter(LP_REAL_ORIENTATION, m_pcAlphabet->GetOrientation());
-  // --
-
-  CSymbolAlphabet alphabet(m_pcAlphabet->GetNumberTextSymbols());
-  alphabet.SetSpaceSymbol(m_pcAlphabet->GetSpaceSymbol());      // FIXME - is this right, or do we have to do some kind of translation?
-  alphabet.SetAlphabetPointer(m_pcAlphabet);    // Horrible hack, but ignore for now.
-
-  // Create an appropriate language model;
-
-  // FIXME - return to using enum here
-
-
-  switch (GetLongParameter(LP_LANGUAGE_MODEL_ID)) {
-  case 0:
-    m_pLanguageModel = new CPPMLanguageModel(m_pEventHandler, m_pSettingsStore, alphabet);
-    break;
-  case 2:
-    m_pLanguageModel = new CWordLanguageModel(m_pEventHandler, m_pSettingsStore, alphabet);
-    break;
-  case 3:
-    m_pLanguageModel = new CMixtureLanguageModel(m_pEventHandler, m_pSettingsStore, alphabet);
-    break;  
-  default:
-    // If there is a bogus value for the language model ID, we'll default
-    // to our trusty old PPM language model.
-    m_pLanguageModel = new CPPMLanguageModel(m_pEventHandler, m_pSettingsStore, alphabet);    
-    break;
-  }
-
-  LearnContext = m_pLanguageModel->CreateEmptyContext();
-
-  // various settings
-  // int iShift = 12;
-  // m_DasherY = 1 << iShift;
-  //  m_DasherOY = m_DasherY / 2;
-  //  m_DasherOX = m_DasherY / 2;
   m_dAddProb = 0.003;
 
-  //  m_Active = CRange(0, GetLongParameter(LP_MAX_Y));
-
   int iNormalization = GetLongParameter(LP_NORMALIZATION);
   m_Rootmin_min = int64_min / iNormalization / 2;
   m_Rootmax_max = int64_max / iNormalization / 2;
 
-  m_pAlphabetManagerFactory = new CAlphabetManagerFactory(this, m_pLanguageModel, bGameMode, strGameModeText);
-  m_pControlManagerFactory = new CControlManagerFactory(this, m_pLanguageModel);
-#ifdef JAPANESE
-  m_pConversionManagerFactory = new CConversionManagerFactory(this, m_pLanguageModel);
-#endif
+  m_pNodeCreationManager = pNCManager;
 
-  m_bContextSensitive = true;
+  m_pLanguageModel = m_pNodeCreationManager->GetLanguageModel();
+  //  LearnContext = m_pNodeCreationManager->GetLearnContext();
+
+  //  m_bContextSensitive = true;
 }
 
 CDasherModel::~CDasherModel() {
@@ -137,18 +90,11 @@ CDasherModel::~CDasherModel() {
     m_Root = NULL;
   }
 
-  delete m_Root;
-
-  delete m_pAlphabetManagerFactory;
-  delete m_pControlManagerFactory;
-
-#ifdef JAPANESE
-  delete m_pConversionManagerFactory;
-#endif
-
-  m_pLanguageModel->ReleaseContext(LearnContext);
-  delete m_pLanguageModel;
+  if(m_Root)
+    delete m_Root;
 
+  //  m_pLanguageModel->ReleaseContext(LearnContext);
+  //  delete m_pLanguageModel;
 }
 
 void CDasherModel::HandleEvent(Dasher::CEvent *pEvent) {
@@ -169,6 +115,12 @@ void CDasherModel::HandleEvent(Dasher::CEvent *pEvent) {
     case BP_DELAY_VIEW:
       MatchTarget(GetBoolParameter(BP_DELAY_VIEW));
       break;
+    case BP_DASHER_PAUSED:
+      if(GetBoolParameter(BP_SLOW_START))
+	m_iStartTime = 0;
+      else
+	m_iStartTime = 1;
+      break;
     default:
       break;
     }
@@ -179,17 +131,8 @@ void CDasherModel::HandleEvent(Dasher::CEvent *pEvent) {
 void CDasherModel::Make_root(CDasherNode *whichchild)
 // find a new root node 
 {
-
-  // TODO - support for this in alphabet manager
-  symbol t = m_Root->Symbol();
-  if(t < m_pDasherInterface->GetAlphabet()->GetNumberTextSymbols()) {
-    // Only learn if we have adaptive behaviour enabled
-
-    // This was m_bAdaptive, which should be a setting for all dasher?
-    if(true)
-      // SYM0
-      m_pLanguageModel->LearnSymbol(LearnContext, t);
-  }
+  // TODO: Need to undo this when root is reparented
+  m_Root->SetFlag(NF_COMMITTED, true);
 
   m_Root->DeleteNephews(whichchild);
 
@@ -256,13 +199,15 @@ void CDasherModel::RebuildAroundNode(CDasherNode *pNode) {
 
 void CDasherModel::Reparent_root(int lower, int upper) {
 
+
   /* Change the root node to the parent of the existing node
      We need to recalculate the coordinates for the "new" root as the 
      user may have moved around within the current root */
 
-  if(m_Root->Symbol() == 0)
-    return; // Don't try to reparent the root symbol
+  // TODO: Reimplement this
 
+//   if(m_Root->Symbol() == 0)
+//     return; // Don't try to reparent the root symbol
 
   CDasherNode *pNewRoot;
 
@@ -298,51 +243,36 @@ void CDasherModel::Reparent_root(int lower, int upper) {
   /* Determine how zoomed in we are */
 
   myint iWidth = upper - lower;
-  
-  m_Root = pNewRoot;
-  
-  myint iRootWidth;
 
+  myint iRootWidth;
   iRootWidth = m_Rootmax - m_Rootmin;
+
+  if(((myint((GetLongParameter(LP_NORMALIZATION) - upper))  / static_cast<double>(iWidth)) > (m_Rootmax_max - m_Rootmax)/static_cast<double>(iRootWidth)) || ((myint(lower) / static_cast<double>(iWidth)) > (m_Rootmin - m_Rootmin_min) / static_cast<double>(iRootWidth))) {
+    pNewRoot->OrphanChild(m_Root);
+    delete pNewRoot;
+    return;
+  }
+
+  m_Root = pNewRoot;
+
   m_Rootmax = m_Rootmax + (myint((GetLongParameter(LP_NORMALIZATION) - upper)) * iRootWidth / iWidth);
   m_Rootmin = m_Rootmin - (myint(lower) * iRootWidth / iWidth);
-  
-  for(std::deque<SGotoItem>::iterator it(m_deGotoQueue.begin()); it != m_deGotoQueue.end(); ++it) {
-    iRootWidth = it->iN2 - it->iN1;
-    it->iN2 = it->iN2 + (myint((GetLongParameter(LP_NORMALIZATION) - upper)) * iRootWidth / iWidth);
-    it->iN1 = it->iN1 - (myint(lower) * iRootWidth / iWidth);
-  }
+
+ for(std::deque<SGotoItem>::iterator it(m_deGotoQueue.begin()); it != m_deGotoQueue.end(); ++it) {
+   iRootWidth = it->iN2 - it->iN1;
+   it->iN2 = it->iN2 + (myint((GetLongParameter(LP_NORMALIZATION) - upper)) * iRootWidth / iWidth);
+   it->iN1 = it->iN1 - (myint(lower) * iRootWidth / iWidth);
+ }
 }
 
-/////////////////////////////////////////////////////////////////////////////
-
 CDasherNode *CDasherModel::Get_node_under_crosshair() {
-//   if(GetBoolParameter(BP_DELAY_VIEW))
-//     return m_Root->Get_node_under(GetLongParameter(LP_NORMALIZATION), m_iTargetMin, m_iTargetMax, GetLongParameter(LP_OX), GetLongParameter(LP_OY));
-//   else
     return m_Root->Get_node_under(GetLongParameter(LP_NORMALIZATION), m_Rootmin + m_iTargetOffset, m_Rootmax + m_iTargetOffset, GetLongParameter(LP_OX), GetLongParameter(LP_OY));
 }
 
-/////////////////////////////////////////////////////////////////////////////
-
 CDasherNode *CDasherModel::Get_node_under_mouse(myint Mousex, myint Mousey) {
-//  if(GetBoolParameter(BP_DELAY_VIEW))
-//     return m_Root->Get_node_under(GetLongParameter(LP_NORMALIZATION), m_iTargetMin, m_iTargetMax, Mousex, Mousey);
-//   else
   return m_Root->Get_node_under(GetLongParameter(LP_NORMALIZATION), m_Rootmin + m_iTargetOffset, m_Rootmax + m_iTargetOffset, Mousex, Mousey);
 }
 
-/////////////////////////////////////////////////////////////////////////////
-
-void CDasherModel::Get_string_under_mouse(const myint Mousex, const myint Mousey, vector <symbol >&str) {
- //  if(GetBoolParameter(BP_DELAY_VIEW))
-//     m_Root->Get_string_under(GetLongParameter(LP_NORMALIZATION), m_iTargetMin, m_iTargetMax, Mousex, Mousey, str);
-//   else
-    m_Root->Get_string_under(GetLongParameter(LP_NORMALIZATION), m_Rootmin + m_iTargetOffset, m_Rootmax + m_iTargetOffset, Mousex, Mousey, str);
-}
-
-/////////////////////////////////////////////////////////////////////////////
-
 void CDasherModel::Start() {
 
   // FIXME - re-evaluate this function and SetContext...
@@ -366,10 +296,10 @@ void CDasherModel::Start() {
 }
 
 void CDasherModel::SetContext(std::string &sNewContext) {
-  
+  // TODO: This is probably the area most significantly in need of a
+  // rethink. Get the language model out of here!  
   m_deGotoQueue.clear();
 
-
   if(oldroots.size() > 0) {
     delete oldroots[0];
     oldroots.clear();
@@ -382,7 +312,7 @@ void CDasherModel::SetContext(std::string &sNewContext) {
   CLanguageModel::Context therootcontext = m_pLanguageModel->CreateEmptyContext();
 
   if(sNewContext.size() == 0) {
-    m_Root = m_pAlphabetManagerFactory->GetRoot(NULL, 0,GetLongParameter(LP_NORMALIZATION), NULL);
+    m_Root = GetRoot(0, NULL, 0,GetLongParameter(LP_NORMALIZATION), NULL);
     
     EnterText(therootcontext, ". ");  
   }
@@ -392,15 +322,17 @@ void CDasherModel::SetContext(std::string &sNewContext) {
     
     int iRootSymbol(vSymbols[vSymbols.size()-1]);
     
-    m_Root = m_pAlphabetManagerFactory->GetRoot(NULL, 0,GetLongParameter(LP_NORMALIZATION), &iRootSymbol);
+    m_Root = GetRoot(0, NULL, 0,GetLongParameter(LP_NORMALIZATION), &iRootSymbol);
     
     EnterText(therootcontext, sNewContext);  
   }
 
-  m_pLanguageModel->ReleaseContext(LearnContext);
-  LearnContext = m_pLanguageModel->CloneContext(therootcontext);
+  // TODO: Reimplement
+  //  m_pLanguageModel->ReleaseContext(LearnContext);
+  // LearnContext = m_pLanguageModel->CloneContext(therootcontext);
 
-  m_Root->SetContext(therootcontext);   // node takes control of the context
+  // TODO: Reimplement
+  //  m_Root->SetContext(therootcontext);   // node takes control of the context
   Recursive_Push_Node(m_Root, 0);
 
   double dFraction( 1 - (1 - m_Root->MostProbableChild() / static_cast<double>(GetLongParameter(LP_NORMALIZATION))) / 2.0 );
@@ -430,18 +362,29 @@ void CDasherModel::SetContext(std::string &sNewContext) {
 ///
 
 void CDasherModel::Get_new_root_coords(myint Mousex, myint Mousey,
-                                         myint &iNewMin, myint &iNewMax) {
+                                         myint &iNewMin, myint &iNewMax, unsigned long iTime) {
   // Comments refer to the code immedialtely before them
 
+  // Avoid Mousex=0, as this corresponds to infinite zoom
   if(Mousex <= 0) {
     Mousex = 1;
   }
 
-  // Avoid Mousex=0, as this corresponds to infinite zoom
+  if(m_iStartTime == 0)
+    m_iStartTime = iTime;
 
-  // If Mousex is too large we risk overflow errors, so make limit it
   int iSteps = m_fr.Steps();
 
+  double dFactor;
+
+  if(IsSlowdown(iTime))
+    dFactor = 0.1 * (1 + 9 * ((iTime - m_iStartTime) / static_cast<double>(GetLongParameter(LP_SLOW_START_TIME))));
+  else 
+    dFactor = 1.0;
+
+  iSteps = static_cast<int>(iSteps / dFactor);
+
+  // If Mousex is too large we risk overflow errors, so limit it
   int iMaxX = (1 << 29) / iSteps;
 
   if(Mousex > iMaxX)
@@ -460,6 +403,8 @@ void CDasherModel::Get_new_root_coords(myint Mousex, myint Mousey,
   // point under the cursor is at the cross-hair. This is where 
   // we want to be in iSteps updates
 
+  //  std::cout << iTargetMin << " " << iTargetMax << std::endl;
+
   DASHER_ASSERT(iSteps > 0);
 
   // iSteps is the number of update steps we need to get the point
@@ -481,7 +426,9 @@ void CDasherModel::Get_new_root_coords(myint Mousex, myint Mousey,
   // expressions are in order to reproduce the behaviour of the old
   // algorithm
 
-  myint iMinSize(m_fr.MinSize(iMaxY));
+  myint iMinSize(m_fr.MinSize(iMaxY, dFactor));
+
+  //  std::cout << iTargetMax - iTargetMin << " " << iMinSize << std::endl;
 
   // Calculate the minimum size of the viewport corresponding to the
   // maximum zoom.
@@ -498,14 +445,7 @@ void CDasherModel::Get_new_root_coords(myint Mousex, myint Mousey,
   iNewMax = (((m_Rootmax - iTargetMax) * (myint)GetLongParameter(LP_MAX_Y)) / (iTargetMax - iTargetMin) + (myint)GetLongParameter(LP_MAX_Y));
 }
 
-bool CDasherModel::Tap_on_display(myint miMousex,
-                                  myint miMousey, 
-                                  unsigned long Time, 
-                                  Dasher::VECTOR_SYMBOL_PROB* pAdded, 
-                                  int* pNumDeleted) 
-        // work out the next viewpoint, opens some new nodes
-{
-  // TODO: Reimplement this 
+bool CDasherModel::UpdatePosition(myint miMousex, myint miMousey, unsigned long iTime, Dasher::VECTOR_SYMBOL_PROB* pAdded, int* pNumDeleted) {
   // Clear out parameters that might get passed in to track user activity
   if (pAdded != NULL)
     pAdded->clear();
@@ -520,7 +460,7 @@ bool CDasherModel::Tap_on_display(myint miMousex,
 
   if(m_deGotoQueue.size() == 0) {
     // works out next viewpoint
-    Get_new_root_coords(miMousex, miMousey, iNewMin, iNewMax);
+    Get_new_root_coords(miMousex, miMousey, iNewMin, iNewMax, iTime);
   
     if(GetBoolParameter(BP_OLD_STYLE_PUSH))
       OldPush(miMousex, miMousey);
@@ -531,10 +471,10 @@ bool CDasherModel::Tap_on_display(myint miMousex,
     m_deGotoQueue.pop_front();
   }
 
-  // Now actually zoom to the new location
-  NewGoTo(iNewMin, iNewMax);
+  m_dTotalNats += log((iNewMax - iNewMin) / static_cast<double>(m_Rootmax - m_Rootmin));
 
-  total_nats += -1.0 * log((iNewMax - iNewMin) / 4096.0);
+  // Now actually zoom to the new location
+  NewGoTo(iNewMin, iNewMax, pAdded, pNumDeleted);
 
   return true;
 }
@@ -623,7 +563,6 @@ double CDasherModel::Plan_new_goto_coords(int iRxnew, myint mousey, int *iSteps,
   DASHER_ASSERT(iRxnew > 0);
   if (iRxnew < ZOOMDENOM && m_Rootmax<(myint)GetLongParameter(LP_MAX_Y) && m_Rootmin>0 ) {
     // refuse to zoom backwards if the entire root node is visible.
-    cout << "Refusing to zoom backwards." << endl;
     *iSteps = 0 ;
     *n1 = m_Rootmin;
     *n2 = m_Rootmax;
@@ -678,17 +617,17 @@ double CDasherModel::Plan_new_goto_coords(int iRxnew, myint mousey, int *iSteps,
   return iRxnew_log;
 }
 
-void CDasherModel::NewGoTo(myint newRootmin, myint newRootmax) {
+void CDasherModel::NewGoTo(myint newRootmin, myint newRootmax, Dasher::VECTOR_SYMBOL_PROB* pAdded, int* pNumDeleted) {
   // Find out the current node under the crosshair
   CDasherNode *old_under_cross=Get_node_under_crosshair();
 
   // Update the max and min of the root node to make iTargetMin and iTargetMax the edges of the viewport.
 
-  if(newRootmin > (myint)GetLongParameter(LP_MAX_Y) / 2 - 1)
-    newRootmin = (myint)GetLongParameter(LP_MAX_Y) / 2 - 1;
+  if(newRootmin + m_iTargetOffset > (myint)GetLongParameter(LP_MAX_Y) / 2 - 1)
+    newRootmin = (myint)GetLongParameter(LP_MAX_Y) / 2 - 1 - m_iTargetOffset;
 
-  if(newRootmax < (myint)GetLongParameter(LP_MAX_Y) / 2 + 1)
-    newRootmax = (myint)GetLongParameter(LP_MAX_Y) / 2 + 1;
+  if(newRootmax + m_iTargetOffset < (myint)GetLongParameter(LP_MAX_Y) / 2 + 1)
+    newRootmax = (myint)GetLongParameter(LP_MAX_Y) / 2 + 1 - m_iTargetOffset;
 
   // Check that we haven't drifted too far. The rule is that we're not
   // allowed to let the root max and min cross the midpoint of the
@@ -720,21 +659,18 @@ void CDasherModel::NewGoTo(myint newRootmin, myint newRootmax) {
   CDasherNode* new_under_cross = Get_node_under_crosshair();
   Push_Node(new_under_cross);
 
-  HandleOutput(new_under_cross, old_under_cross);
+  HandleOutput(new_under_cross, old_under_cross, pAdded, pNumDeleted);
+}
 
- }
-
-void CDasherModel::HandleOutput(CDasherNode *pNewNode, CDasherNode *pOldNode) {
-  if(pNewNode == pOldNode)
-    return;
-
-  DeleteCharacters(pNewNode, pOldNode);
+void CDasherModel::HandleOutput(CDasherNode *pNewNode, CDasherNode *pOldNode, Dasher::VECTOR_SYMBOL_PROB* pAdded, int* pNumDeleted) {
+  if(pNewNode != pOldNode)
+    DeleteCharacters(pNewNode, pOldNode, pNumDeleted);
   
   if(pNewNode->isSeen())
     return;
 
   // TODO: Reimplement second parameter
-  RecursiveOutput(pNewNode, NULL);
+  RecursiveOutput(pNewNode, pAdded);
 }
 
 
@@ -784,10 +720,6 @@ bool CDasherModel::DeleteCharacters(CDasherNode *newnode, CDasherNode *oldnode,
   else {
     // This one's more complicated - the user may have moved onto a new branch
     // Find the last seen node on the new branch
-
-    // Note that it's possible here that it just corresponds to moving
-    // forwards rather than backwards. In this case we should get
-    // lastseen == oldnode, hence we don't do anything in the latter loop.
     CDasherNode *lastseen = newnode->Parent();
 
     while(lastseen != NULL && lastseen->isSeen() == false) {
@@ -800,9 +732,6 @@ bool CDasherModel::DeleteCharacters(CDasherNode *newnode, CDasherNode *oldnode,
       
       oldnode->m_pNodeManager->Undo(oldnode);
 
-      // TODO: This is a hack. We really need to think through the
-      // node flags and be absolutely sure that there's nothing
-      // untoward. (cf. bug #370478).
       if(oldnode->Parent())
 	oldnode->Parent()->m_pNodeManager->Enter(oldnode->Parent());
 
@@ -819,104 +748,24 @@ bool CDasherModel::DeleteCharacters(CDasherNode *newnode, CDasherNode *oldnode,
 
 /////////////////////////////////////////////////////////////////////////////
 
-// Diagnostic trace
-void CDasherModel::Trace() const {
-  // OutputDebugString(TEXT(" ptr   symbol   context Next  Child    pushme pushed cscheme   lbnd  hbnd \n"));
-  m_Root->Trace();
-}
+// // Diagnostic trace
+// void CDasherModel::Trace() const {
+//   // OutputDebugString(TEXT(" ptr   symbol   context Next  Child    pushme pushed cscheme   lbnd  hbnd \n"));
+//   m_Root->Trace();
+// }
 
 ///////////////////////////////////////////////////////////////////
 
 void CDasherModel::GetProbs(CLanguageModel::Context context, vector <symbol >&NewSymbols, vector <unsigned int >&Probs, int iNorm) const {
-  // Total number of symbols
-  int iSymbols = m_pcAlphabet->GetNumberSymbols();      // note that this includes the control node and the root node
-
-  // Number of text symbols, for which the language model gives the distribution
-  // int iTextSymbols = m_pcAlphabet->GetNumberTextSymbols();
-  
-  NewSymbols.resize(iSymbols);
-//      Groups.resize(iSymbols);
-  for(int i = 0; i < iSymbols; i++) {
-    NewSymbols[i] = i;          // This will be replaced by something that works out valid nodes for this context
-    //      Groups[i]=m_pcAlphabet->get_group(i);
-  }
-
-  // TODO - sort out size of control node - for the timebeing I'll fix the control node at 5%
-
-  int uniform_add;
-  int nonuniform_norm;
-  int control_space;
-  int uniform = GetLongParameter(LP_UNIFORM);
-
-  if(!GetBoolParameter(BP_CONTROL_MODE)) {
-    control_space = 0;
-    uniform_add = ((iNorm * uniform) / 1000) / (iSymbols - 2);  // Subtract 2 from no symbols to lose control/root nodes
-    nonuniform_norm = iNorm - (iSymbols - 2) * uniform_add;
-  }
-  else {
-    control_space = int (iNorm * 0.05);
-    uniform_add = (((iNorm - control_space) * uniform / 1000) / (iSymbols - 2));        // Subtract 2 from no symbols to lose control/root nodes
-    nonuniform_norm = iNorm - control_space - (iSymbols - 2) * uniform_add;
-  }
-
-  m_pLanguageModel->GetProbs(context, Probs, nonuniform_norm);
-
-#if _DEBUG
-  int iTotal = 0;
-  for(int k = 0; k < Probs.size(); ++k)
-    iTotal += Probs[k];
-  DASHER_ASSERT(iTotal == nonuniform_norm);
-#endif
-
-  //  Probs.insert(Probs.begin(), 0);
-
-  for(unsigned int k(1); k < Probs.size(); ++k)
-    Probs[k] += uniform_add;
-
-  Probs.push_back(control_space);
-
-#if _DEBUG
-  iTotal = 0;
-  for(int k = 0; k < Probs.size(); ++k)
-    iTotal += Probs[k];
-//      DASHER_ASSERT(iTotal == iNorm);
-#endif
-
+  m_pNodeCreationManager->GetProbs(context, NewSymbols, Probs, iNorm);
 }
 
 void CDasherModel::LearnText(CLanguageModel::Context context, string *TheText, bool IsMore) {
-  vector < symbol > Symbols;
-
-  m_pcAlphabet->GetSymbols(&Symbols, TheText, IsMore);
-
-  for(unsigned int i = 0; i < Symbols.size(); i++)
-    m_pLanguageModel->LearnSymbol(context, Symbols[i]); // FIXME - conversion to symbol alphabet
+  m_pNodeCreationManager->LearnText(context, TheText, IsMore);
 }
 
 void CDasherModel::EnterText(CLanguageModel::Context context, string TheText) const {
-  vector < symbol > Symbols;
-  m_pcAlphabet->GetSymbols(&Symbols, &TheText, false);
-  for(unsigned int i = 0; i < Symbols.size(); i++)
-    m_pLanguageModel->EnterSymbol(context, Symbols[i]); // FIXME - conversion to symbol alphabet
-}
-
-CDasherModel::CTrainer::CTrainer(CDasherModel &DasherModel)
-:m_DasherModel(DasherModel) {
-  m_Context = m_DasherModel.m_pLanguageModel->CreateEmptyContext();
-}
-
-void CDasherModel::CTrainer::Train(const std::vector<symbol> &vSymbols) {
-  for(std::vector<symbol>::const_iterator it(vSymbols.begin()); it != vSymbols.end(); ++it)
-    m_DasherModel.m_pLanguageModel->LearnSymbol(m_Context, *it);
-}
-
-CDasherModel::CTrainer::~CTrainer() {
-  m_DasherModel.m_pLanguageModel->ReleaseContext(m_Context);
-
-}
-
-CDasherModel::CTrainer * CDasherModel::GetTrainer() {
-  return new CDasherModel::CTrainer(*this);
+  m_pNodeCreationManager->EnterText(context, TheText);
 }
 
 void CDasherModel::Push_Node(CDasherNode *pNode) {
@@ -930,34 +779,10 @@ void CDasherModel::Push_Node(CDasherNode *pNode) {
     return;
   }
 
+  // TODO: Do we really need to delete all of the children at this point?
   pNode->Delete_children();
 
-  // This ASSERT seems to routinely fail
-  //DASHER_ASSERT(pNode->Symbol()!=0);
-
-  // if we haven't got a context then derive it
-
-  if(!pNode->Context()) {
-    CLanguageModel::Context cont;
-    // sym0
-    if(pNode->Symbol() < m_pcAlphabet->GetNumberTextSymbols() && pNode->Symbol() > 0) {
-      CDasherNode *pParent = pNode->Parent();
-      DASHER_ASSERT(pParent != NULL);
-      // Normal symbol - derive context from parent
-      cont = m_pLanguageModel->CloneContext(pParent->Context());
-      m_pLanguageModel->EnterSymbol(cont, pNode->Symbol());
-    } else {
-      // For new "root" nodes (such as under control mode), we want to 
-      // mimic the root context
-      cont = CreateEmptyContext();
-      //      EnterText(cont, "");
-    }
-    pNode->SetContext(cont);
-
-  }
-
   pNode->Alive(true);
-
   pNode->m_pNodeManager->PopulateChildren(pNode);
   pNode->SetHasAllChildren(true);
 }
@@ -968,10 +793,6 @@ void CDasherModel::Recursive_Push_Node(CDasherNode *pNode, int iDepth) {
     return;
   }
 
- //  if(pNode->Symbol() == GetControlSymbol()) {
-//     return;
-//   }
-
   Push_Node(pNode);
 
   if(iDepth == 0)
@@ -1028,7 +849,7 @@ bool CDasherModel::CheckForNewRoot(CDasherView *pView) {
 
   CDasherNode *root(m_Root);
   CDasherNode::ChildMap & children = m_Root->Children();
-  
+
   if(pView->IsNodeVisible(m_Rootmin,m_Rootmax)) {
     Reparent_root(root->Lbnd(), root->Hbnd());
     return(m_Root != root);
@@ -1039,7 +860,6 @@ bool CDasherModel::CheckForNewRoot(CDasherView *pView) {
 
   int alive = 0;
   CDasherNode *theone = 0;
-
  
   // Find whether there is exactly one alive child; if more, we don't care.
   CDasherNode::ChildMap::iterator i;
@@ -1090,7 +910,7 @@ void CDasherModel::ScheduleZoom(int dasherx, int dashery) {
 
    double dCFactor(CorrectionFactor(dasherx, dashery));
 
-   int iSteps = GetLongParameter(LP_ZOOMSTEPS) * dCFactor;
+   int iSteps = static_cast<int>(GetLongParameter(LP_ZOOMSTEPS) * dCFactor);
   // myint iRxnew = ((GetLongParameter(LP_OX)/2) * GetLongParameter(LP_OX)) / dasherx;
    myint n1, n2, iTarget1, iTarget2;
    //Plan_new_goto_coords(iRxnew, dashery, &iSteps, &o1,&o2,&n1,&n2);
@@ -1100,8 +920,8 @@ void CDasherModel::ScheduleZoom(int dasherx, int dashery) {
 
    double dZ = 4096 / static_cast<double>(iTarget2 - iTarget1);
 
-   n1 = (m_Rootmin - iTarget1) * dZ;
-   n2 = (m_Rootmax - iTarget2) * dZ + 4096;
+   n1 = static_cast<int>((m_Rootmin - iTarget1) * dZ);
+   n2 = static_cast<int>((m_Rootmax - iTarget2) * dZ + 4096);
  
    m_deGotoQueue.clear();
    SGotoItem sNewItem;
@@ -1138,13 +958,14 @@ void CDasherModel::MatchTarget(bool bReverse) {
   m_Rootmax += m_iTargetOffset;
 
   m_iTargetOffset = 0;
+}
 
-//   if(bReverse) {
-//     m_iTargetMin = m_Rootmin;
-//     m_iTargetMax = m_Rootmax;
-//   }
-//   else {
-//     m_Rootmin = m_iTargetMin;
-//     m_Rootmax = m_iTargetMax;
-//   }
+void CDasherModel::LimitRoot(int iMaxWidth) {
+  m_Rootmin = GetLongParameter(LP_MAX_Y) / 2 - iMaxWidth / 2;
+  m_Rootmax = GetLongParameter(LP_MAX_Y) / 2 + iMaxWidth / 2;
+}
+
+
+CAlphabetManagerFactory::CTrainer *CDasherModel::GetTrainer() {
+  return m_pNodeCreationManager->GetTrainer();
 }
