@@ -17,13 +17,10 @@
 #include "DasherNode.h"
 #include "DasherComponent.h"
 #include "Alphabet/Alphabet.h"
-#include "Alphabet/AlphIO.h"
 #include "AlphabetManagerFactory.h"
 #include "ControlManagerFactory.h"
-
-#ifdef JAPANESE
 #include "ConversionManagerFactory.h"
-#endif
+#include "NodeCreationManager.h"
 
 #include <math.h>
 #include "DasherTypes.h"
@@ -31,8 +28,6 @@
 #include <vector>
 #include <deque>
 
-
-
 namespace Dasher {
   class CDasherModel;
   class CDasherInterfaceBase;
@@ -51,19 +46,9 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
 {
  public:
 
-  class CTrainer {
-  public:
-    CTrainer(CDasherModel & DasherModel);
-
-    void Train(const std::vector < symbol > &vSymbols);
-
-    ~CTrainer();
-
-  private:
-    CLanguageModel::Context m_Context;
-    CDasherModel & m_DasherModel;
-
-  };
+  /// 
+  /// Member class used to train the language model
+  ///
 
   typedef enum {
     idPPM = 0,
@@ -73,9 +58,18 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
     idJapanese = 4
   } LanguageModelID;
 
-  CDasherModel(CEventHandler * pEventHandler, CSettingsStore * pSettingsStore, CDasherInterfaceBase * pDashIface, CAlphIO *pAlphIO, bool bGameMode = false, const std::string &strGameModeText = "");
+  CDasherModel(CEventHandler * pEventHandler, CSettingsStore * pSettingsStore, CNodeCreationManager *pNCManager, CDasherInterfaceBase * pDashIface, bool bGameMode = false, const std::string &strGameModeText = "");
   ~CDasherModel();
 
+  /// @name Language model passthroughs 
+  /// These functions should be made obsolete - we can refer to the
+  /// language model now without passing through here.
+  /// @{
+
+  ///
+  /// Prototype binary dump of language model data
+  ///
+
   bool WriteLMToFile(const std::string &strFilename) {
     if(m_pLanguageModel)
       return m_pLanguageModel->WriteToFile(strFilename);
@@ -83,6 +77,10 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
       return false;
   }
 
+  ///
+  /// Prototype binary read of language model data
+  ///
+
   bool ReadLMFromFile(const std::string &strFilename) {
     if(m_pLanguageModel)
       return m_pLanguageModel->ReadFromFile(strFilename);
@@ -90,76 +88,167 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
       return false;
   }
 
+  ///
+  /// Update a language model context to include an additional character
+  ///
+
+  void EnterText(CLanguageModel::Context Context, std::string TheText) const;
+
+  ///
+  /// Update a language model context to include an additional
+  /// character, also learning the new data.
+
+  void LearnText(CLanguageModel::Context Context, std::string * TheText, bool IsMore);
+
+  ///
+  /// Create an empty language model context
+  ///
+
+  CLanguageModel::Context CreateEmptyContext() const;
+
+  ///
+  /// Return a trainer object for the language model
+  ///
+
+  CAlphabetManagerFactory::CTrainer *GetTrainer();
+
+  void GetProbs(CLanguageModel::Context context, std::vector < symbol > &NewSymbols, std::vector < unsigned int >&Probs, int iNorm) const;
+
+  /// @}
+
+  /// @name Alphabet passthroughs
+  /// Should be obsolete - we shound't need to know about alphabets
+  /// outside of the node manager
+  /// @{
+
+  ///
+  /// Get the symbol ID representing space
+  ///
+
+  symbol GetSpaceSymbol() const {
+    return m_pNodeCreationManager->GetSpaceSymbol();
+  }
+
+  ///
+  /// Get the symbol ID representing the control node 
+  ///
+
+  symbol GetControlSymbol() const {
+    return m_pNodeCreationManager->GetControlSymbol();
+  }
+
+  ///
+  /// Get the symbol ID representing the conversion pseudo-character
+  ///
+
+  symbol GetStartConversionSymbol() const {
+    return m_pNodeCreationManager->GetStartConversionSymbol();
+  }
+
+  ///
+  /// Convert a given symbol ID to display text
+  ///
+
+  const std::string & GetDisplayText(int iSymbol) const {
+    return m_pNodeCreationManager->GetDisplayText(iSymbol);
+  }
+
+  int GetColour(symbol s) const;
+
+  /// @}
+
+  ///
+  /// Event handler
+  ///
+
   void HandleEvent(Dasher::CEvent * pEvent);
 
-  CTrainer *GetTrainer();
+  /// @name Dymanic evolution
+  /// Routines detailing the timer dependent evolution of the model
+  /// @{
+
+  ///
+  /// Update the root location - called in response to regular timer
+  /// callbacks
+  ///
+
+  bool UpdatePosition(myint, myint, unsigned long iTime, Dasher::VECTOR_SYMBOL_PROB* pAdded = NULL, int* pNumDeleted = NULL);  
+  ///
+  /// Notify the framerate class that a new frame has occurred
+  ///
 
-  // framerate functions
   void NewFrame(unsigned long Time) {
     m_fr.NewFrame(Time);
   }
 
-  // called everytime we render a new frame
+  ///
+  /// Apply an offset to the 'target' coordinates - implements the jumps in
+  /// two button dynamic mode.
+  ///
+
+  void Offset(int iOffset);
+ 
+  ///
+  /// Reset the 'target' root coordinates to match those currently visible. 
+  /// Appropriate for abrubt changes in behaviour (such as backing off in 
+  /// button modes)
+  /// 
+
+  void MatchTarget(bool bReverse);
+
+  /// @}
+
+  ///
+  /// Get the current framerate
+  ///
+
   double Framerate() const {
     return m_fr.Framerate();
   }
 
+  ///
+  /// Reset the framerate class
+  ///
+
   void Reset_framerate(unsigned long Time) {
     m_fr.Reset(Time);
   }
 
+  ///
+  /// Initialise the framerate class - presumably called whenever
+  /// dasher is stoppe, but the actual semantics here need to be
+  /// verified
+  ///
+
   void Halt() {
     m_fr.Initialise();
   }
 
-  void RecursiveOutput(CDasherNode *pNode, Dasher::VECTOR_SYMBOL_PROB* pAdded);
-
-  // User control of speed
+  ///
+  /// Set the target bitrate - probably shouldn't be called externally
+  /// - could implement through the event subsystem instead
+  ///
+  
   void SetBitrate(double TargetRate) {
     m_fr.SetBitrate(TargetRate);
-  }                             // Use or start at this bitrate
-  //void SetMaxBitrate(double MaxRate) {m_dMaxRate=MaxRate;m_fr.SetMaxBitrate(MaxRate);} // Cap any adaption at this rate
+  } 
 
-/*   std::string GroupLabel(int group) const { */
-/*     return m_pcAlphabet->GetGroupLabel(group); */
-/*   } */
-/*   int GroupColour(int group) const { */
-/*     return m_pcAlphabet->GetGroupColour(group); */
-/*   } */
-
-  void SetContext(std::string & sNewContext);
-
-  // functions returning private data (read only access)
-  myint Rootmin() const {
-    return m_Rootmin;
-  }
-  myint Rootmax() const {
-    return m_Rootmax;
-  }
-  CDasherNode *Root() const {
-    return m_Root;
-  }
-
-  void OutputCharacters(CDasherNode * node);
-  bool DeleteCharacters(CDasherNode * newnode, CDasherNode * oldnode, int* pNumDeleted = NULL);
-
-  void Trace() const;           // diagnostics
-  //void Learn_symbol(symbol Symbol) {m_languagemodel->learn_symbol(Symbol);} // feed character to language model
-
-  bool Tap_on_display(myint, myint, unsigned long Time, Dasher::VECTOR_SYMBOL_PROB* pAdded = NULL, int* pNumDeleted = NULL);  // evolves the current viewpoint
-
-/*   void GoTo(double, myint);     // jumps to a new viewpoint */
-   void NewGoTo(myint n1, myint n2);
-   void OldPush(myint iMousex, myint iMousey);
-   double Plan_new_goto_coords(int iRxnew, myint mousey, int *iSteps, myint *o1, myint *o2 , myint *n1, myint *n2); 
-
-  void Start();                 // initializes the data structure
-
-  /// 
-  /// Make a child of the root into a new root
+  ///
+  /// Set the context in which predictions are made - need to check
+  /// the semantics here. Probably a sensible thing to do here is to
+  /// not allow context changes within an existing model, but to force
+  /// a rebuild. This will only work once the language model has been
+  /// decoupled, as retraining every time would be somewhat
+  /// inconvenient.
   ///
 
-  void Make_root(CDasherNode *whichchild); 
+  void SetContext(std::string & sNewContext);
+  
+  ///
+  /// Check semantics here
+  ///
+
+  void Start();                 // initializes the data structure
 
   ///
   /// Clear the queue of old roots - used when those nodes become
@@ -169,124 +258,109 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
   void ClearRootQueue();
 
   ///
-  /// A version of Make_root which is suitable for arbitrary
-  /// descendents of the root, not just immediate children.
+  /// Reset counter of total nats entered
   ///
-
-  void RecursiveMakeRoot(CDasherNode *pNewRoot);
-
-  ///
-  /// Rebuild the data structure such that a given node is guaranteed
-  /// to be in the same place on the screen. This would usually be the
-  /// node under the crosshair
-  ///
-
-  void RebuildAroundNode(CDasherNode *pNode);
-
-  void Reparent_root(int lower, int upper);     // change back to the previous root
-
  
   void ResetNats() {
-    total_nats = 0;
+    m_dTotalNats = 0.0;
   }
 
+  ///
+  /// Return the total nats entered
+  ///
+
   double GetNats() {
-    return total_nats;
-  }
-
-//  myint PlotGoTo(myint MouseX, myint MouseY);
-
-  void EnterText(CLanguageModel::Context Context, std::string TheText) const;
-  void LearnText(CLanguageModel::Context Context, std::string * TheText, bool IsMore);
-
-  CLanguageModel::Context CreateEmptyContext()const;
-
-  // Alphabet pass-through functions for widely needed information
-  symbol GetSpaceSymbol() const {
-    return m_pcAlphabet->GetSpaceSymbol();
-  }
-
-  symbol GetControlSymbol() const {
-    return m_pcAlphabet->GetControlSymbol();
-  }
-
-  symbol GetStartConversionSymbol() const {
-    return m_pcAlphabet->GetStartConversionSymbol();
+    return m_dTotalNats;
   }
 
 
-  const std::string & GetDisplayText(int iSymbol) const {
-    return m_pcAlphabet->GetDisplayText(iSymbol);
-  }
+  ///
+  /// Get a root node of a given type
+  ///
 
-  const CAlphabet & GetAlphabet() const {
-    return *m_pcAlphabet;
-  }
-
-  CAlphabet *GetAlphabetNew() const {
-    return m_pcAlphabet;
-  }
-
-  CDasherNode *Get_node_under_crosshair();    // Needed for Game Mode
-  
   CDasherNode *GetRoot( int iType, CDasherNode *pParent, int iLower, int iUpper, void *pUserData ) {
-    switch(iType) {
-    case 0:
-      return m_pAlphabetManagerFactory->GetRoot(pParent, iLower, iUpper, pUserData);
-    case 1:
-      return m_pControlManagerFactory->GetRoot(pParent, iLower, iUpper, pUserData);
-#ifdef JAPANESE
-    case 2:
-      return m_pConversionManagerFactory->GetRoot(pParent, iLower, iUpper, pUserData);
-#endif
-    }
+    return m_pNodeCreationManager->GetRoot(iType, pParent, iLower, iUpper, pUserData);
   };
   
-  // FIXME - only public temporarily
-  void GetProbs(CLanguageModel::Context context, std::vector < symbol > &NewSymbols, std::vector < unsigned int >&Probs, int iNorm) const;
-  int GetColour(symbol s) const;
 
-  
-  // Control mode stuff
-
-  void RegisterNode( int iID, const std::string &strLabel, int iColour ) {
-    m_pControlManagerFactory->RegisterNode(iID, strLabel, iColour);
-  }
-  
-  void ConnectNode(int iChild, int iParent, int iAfter) {
-     m_pControlManagerFactory->ConnectNode(iChild, iParent, iAfter);
-  }
-
-  void DisconnectNode(int iChild, int iParent) {
-     m_pControlManagerFactory->DisconnectNode(iChild, iParent);
-  }
-
-  void Push_Node(CDasherNode * pNode);  // give birth to children
-
-  bool m_bContextSensitive;
+  ///
+  /// TODO: Move these elsewhere
+  ///
 
+  //  bool m_bContextSensitive;
   std::string m_strContextBuffer;
 
+  /// 
+  /// @name Rendering
+  /// Methods to do with rendering the model to a view
+  /// @{
+
+  /// 
+  /// Render the model to a given view
+  ///
+
   bool RenderToView(CDasherView *pView, bool bRedrawDisplay);
 
-  bool CheckForNewRoot(CDasherView *pView);
+  /// @}
+
+  /// 
+  /// @name Scheduled operation
+  /// E.g. response to button mode
+  /// @{
+
+  ///
+  /// Schedule zoom to a given Dasher coordinate (used in click mode,
+  /// button mode etc.)
+  ///
 
   void ScheduleZoom(int dasherx, int dashery);
 
-  void HandleOutput(CDasherNode *pNewNode, CDasherNode *pOldNode);
+  ///
+  /// Return the number of remaining zoom steps
+  ///
 
   int ScheduledSteps() {
     return m_deGotoQueue.size();
   }
 
-  // Apply an offset to the 'target' coordinates - implements the jumps in
-  // two button dynamic mode.
-  void Offset(int iOffset);
- 
-  // Reset the 'target' root coordinates to match those currently visible. 
-  // Appropriate for abrubt changes in behaviour (such as backing off in 
-  // button modes)
-  void MatchTarget(bool bReverse);
+  /// @}
+
+  ///
+  /// This is pretty horrible - a rethink of the start/reset mechanism
+  /// is definitely in order. Used to prevent the root node from being
+  /// too large in various modes before Dasher is started.
+  ///
+
+  void LimitRoot(int iMaxWidth);
+
+  ///
+  /// Cause Dasher to temporarily slow down (eg as part of automatic
+  /// speed control in n-button dynamic mode).
+  ///
+
+  void TriggerSlowdown() {
+    m_iStartTime = 0;
+  };
+
+  ///
+  /// Return whether Dasher is currently being temporarily slowed or
+  /// not.
+  ///
+
+  bool IsSlowdown(unsigned long iTime) {
+    return ((iTime - m_iStartTime) < static_cast<unsigned long>(GetLongParameter(LP_SLOW_START_TIME)));
+  };
+
+  ///
+  /// Check whether a change of root node is needed, and perform the
+  /// update if so
+  ///
+
+  bool CheckForNewRoot(CDasherView *pView);
+
+ protected:
+  int m_iRenderCount;
+
 
  private:
 
@@ -298,8 +372,6 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
 
   CLanguageModel *m_pLanguageModel;     // pointer to the language model
 
-  CAlphabet *m_pcAlphabet;        // pointer to the alphabet
-
   CLanguageModel::Context LearnContext; // Used to add data to model as it is entered
 
   /////////////////////////////////////////////////////////////////////////////
@@ -324,7 +396,7 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
 
   CFrameRate m_fr;              // keep track of framerate
 
-  double total_nats;            // Information entered so far
+  double m_dTotalNats;            // Information entered so far
 
   // the probability that gets added to every symbol
   double m_dAddProb;
@@ -335,26 +407,99 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
 
   CDasherNode *Get_node_under_mouse(myint smousex, myint smousey);
 
-  void Get_new_root_coords(myint mousex, myint mousey, myint &iNewMin, myint &iNewMax);
-
- // void Get_new_goto_coords(double zoomfactor, myint mousey);
-
-  void Get_string_under_mouse(const myint smousex, const myint smousey, std::vector < symbol > &str);
+  void Get_new_root_coords(myint mousex, myint mousey, myint &iNewMin, myint &iNewMax, unsigned long iTime);
 
   double CorrectionFactor(int dasherx, int dashery);
 
   void Recursive_Push_Node(CDasherNode * pNode, int depth);
 
-  //  ControlTree *m_pControltree;
+  ///
+  /// Perform output on a node - recurse up the tree outputting any
+  /// symbols which have not been output so far. Neeed to check
+  /// behaviour with respect to deletion
+  ///
 
-  CAlphabetManagerFactory *m_pAlphabetManagerFactory;
-  CControlManagerFactory *m_pControlManagerFactory;
+  void RecursiveOutput(CDasherNode *pNode, Dasher::VECTOR_SYMBOL_PROB* pAdded);
 
+
+  ///
+  /// Go directly to a given coordinate - check semantics
+  ///
+  
+  void NewGoTo(myint n1, myint n2, Dasher::VECTOR_SYMBOL_PROB* pAdded, int* pNumDeleted);
+
+  ///
+  /// Check semantics here
+  ///
+
+  void OutputCharacters(CDasherNode * node);
+  bool DeleteCharacters(CDasherNode * newnode, CDasherNode * oldnode, int* pNumDeleted = NULL);
+
+  ///
+  /// Create the children of a Dasher node
+  ///
+
+  void Push_Node(CDasherNode * pNode); 
+
+  /// 
+  /// Old style drilling down of nodes - optionally can still be
+  /// called
+  ///
+
+  void OldPush(myint iMousex, myint iMousey);
+
+  ///
+  /// Schedule smooth transition to a given coordinate
+  ///
+
+  double Plan_new_goto_coords(int iRxnew, myint mousey, int *iSteps, myint *o1, myint *o2 , myint *n1, myint *n2); 
+
+  /// 
+  /// Make a child of the root into a new root
+  ///
+
+  void Make_root(CDasherNode *whichchild); 
+
+  ///
+  /// A version of Make_root which is suitable for arbitrary
+  /// descendents of the root, not just immediate children.
+  ///
+
+  void RecursiveMakeRoot(CDasherNode *pNewRoot);
+
+  ///
+  /// Rebuild the data structure such that a given node is guaranteed
+  /// to be in the same place on the screen. This would usually be the
+  /// node under the crosshair
+  ///
+
+  void RebuildAroundNode(CDasherNode *pNode);
+
+  ///
+  /// Rebuild the parent of the current root - used during backing off
+  ///
+
+  void Reparent_root(int lower, int upper); 
+
+  ///
+  /// Return a pointer to the Dasher node which is currently under the
+  /// crosshair. Apparently this is needed for game mode.
+  ///
+
+  CDasherNode *Get_node_under_crosshair();    
+
+  ///
+  /// Handle the output caused by a change in node being over the
+  /// crosshair
+  ///
+
+  void HandleOutput(CDasherNode *pNewNode, CDasherNode *pOldNode, Dasher::VECTOR_SYMBOL_PROB* pAdded, int* pNumDeleted);
+
+  CNodeCreationManager *m_pNodeCreationManager;
   bool m_bGameMode;
 
-#ifdef JAPANESE
-  CConversionManagerFactory *m_pConversionManagerFactory;
-#endif
+
+  unsigned long m_iStartTime;
 
   struct SGotoItem {
     myint iN1;
@@ -369,8 +514,8 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
   // TODO: Need to rethink this at some point.
   bool m_bRequireConversion;
 
-  friend class CTrainer;
-  friend class CDasherNode;
+  //  friend class CTrainer;
+  //  friend class CDasherNode;
 
 };
 
@@ -380,8 +525,8 @@ inline CLanguageModel::Context CDasherModel::CreateEmptyContext() const {
   return m_pLanguageModel->CreateEmptyContext();
 }
 
-inline int CDasherModel::GetColour(symbol s) const {
-  return m_pcAlphabet->GetColour(s);
+inline int CDasherModel::GetColour(symbol s) const { 
+  return m_pNodeCreationManager->GetColour(s); 
 }
 
 #endif /* #ifndef __DasherModel_h__ */
