@@ -241,7 +241,7 @@ int CDasherViewSquare::RenderNode(const int Color, myint y1, myint y2, int &most
   Dasher2Screen(0, std::max(y1, iDasherMinY), iScreenX1, iScreenY1);
   Dasher2Screen(0, std::min(y2, iDasherMaxY), iScreenX2, iScreenY2);
 
-  Cint32 iHeight = std::max(iScreenY2 - iScreenY1, 0);
+  Cint32 iHeight = std::max(iScreenY2 - iScreenY1, (screenint)0);
 
   if(iHeight <= 1)
     return 0;                   // We're too small to render
@@ -561,7 +561,7 @@ void CDasherViewSquare::VisibleRegion( myint &iDasherMinX, myint &iDasherMinY, m
   iDasherMaxY = m_iDasherMaxY;
 }
 
-void CDasherViewSquare::GetCoordinates(unsigned long Time, myint &iDasherX, myint &iDasherY) {
+int CDasherViewSquare::GetCoordinates(unsigned long Time, myint &iDasherX, myint &iDasherY) {
 
   // FIXME - Actually turn autocalibration on and off!
   // FIXME - AutoCalibrate should use Dasher co-ordinates, not raw mouse co-ordinates?
@@ -599,12 +599,13 @@ void CDasherViewSquare::GetCoordinates(unsigned long Time, myint &iDasherX, myin
 
   // Convert the input co-ordinates to dasher co-ordinates
 
-  int mode;
-  
-  mode = 0;
- 
   //  Input2Dasher(mousex, mousey, iDasherX, iDasherY);
-  Screen2Dasher(mousex, mousey, iDasherX, iDasherY, false, true );
+  if(iType == 0)
+    Screen2Dasher(mousex, mousey, iDasherX, iDasherY, false, true );
+  else {
+    iDasherX = mousex;
+    iDasherY = mousey;
+  }
 
 //   m_iDasherXCache = iDasherX;
 //   m_iDasherYCache = iDasherY;
@@ -618,6 +619,8 @@ void CDasherViewSquare::GetCoordinates(unsigned long Time, myint &iDasherX, myin
   //  iDasherX = myint(xmap(iDasherX / static_cast < double >(GetLongParameter(LP_MAX_Y))) * GetLongParameter(LP_MAX_Y));
   // iDasherY = m_ymap.map(iDasherY);
 #endif
+
+  return iType;
 }
 
 void CDasherViewSquare::NewDrawGoTo(myint iDasherMin, myint iDasherMax, bool bActive) {
