@@ -3,7 +3,6 @@
 
 #include "../DasherCore/DasherScreen.h"
 #include "../DasherCore/DasherTypes.h"
-#include "../DasherCore/CustomColours.h"
 
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
@@ -156,7 +155,7 @@ public:
 
   void DrawRectangle(screenint x1, screenint y1, screenint x2, screenint y2, int Color, int iOutlineColour, Opts::ColorSchemes ColorScheme, bool bDrawOutine, bool bFill, int iThickness);
 
-  void DrawCircle(screenint iCX, screenint iCY, screenint iR, int iColour, bool bFill);
+  void DrawCircle(screenint iCX, screenint iCY, screenint iR, int iColour, int iFillColour, int iThickness, bool bFill);
 
   ///
   /// Send a marker to indicate phases of the redraw process. This is
@@ -213,7 +212,7 @@ public:
   /// \param Colours New colours to use
   ///
 
-  void SetColourScheme(const CCustomColours *Colours);
+  void SetColourScheme(const CColourIO::ColourInfo *pColourScheme);
 
   /// 
   /// Gets the location and size of our canvas.
@@ -240,6 +239,16 @@ private:
 
   GtkWidget *m_pCanvas;
 
+#if WITH_CAIRO
+
+  cairo_surface_t *m_pDisplaySurface;
+  cairo_surface_t *m_pDecorationSurface;
+  cairo_surface_t *m_pOnscreenSurface;
+
+  cairo_surface_t *m_pOffscreenbuffer;
+
+#else
+
   ///
   /// The offscreen buffer containing the 'background'
   ///
@@ -267,6 +276,8 @@ private:
   GdkPixmap *m_pOffscreenBuffer;
   GdkPixmap *m_pDummyBuffer;
 
+#endif
+
   /// 
   /// The Pango cache - used to store pre-computed pango layouts as
   /// they are costly to regenerate every time they are needed.
@@ -280,17 +291,13 @@ private:
 
   PangoRectangle *m_pPangoInk;
 
-  ///
-  /// The signal handler ID for the expose callback - stored so it can
-  /// be disconnected when the CCanvas object is destroyed (which will
-  /// happen whenever the canvas is resized).
-  ///
-
-  gulong lSignalHandler;
-
 #if WITH_CAIRO
   cairo_t *display_cr;
   cairo_t *decoration_cr;
+  cairo_t *onscreen_cr; // TODO: do we need to do our own double buffering?
+
+  cairo_t *widget_cr;
+
   cairo_t *cr;
   my_cairo_colour_t *cairo_colours;
 #else
